/*******************************************************************************
*    Robot "Giskard" 1.0.                                                      *
*    Copyright (C) 2009 Luca Bertoncello                                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    Circuits by Marian Neubert marian@tesla-crew.de                           *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU General Public License as published by      *
*    the Free Software Foundation; version 2 of the License.                   *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU General Public License for more details.                              *
*                                                                              *
*    You should have received a copy of the GNU General Public License         *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

#define UART_BAUD_RATE    9600

#define FALSE             0
#define TRUE              1
#define MYPROTOCOLVERSION 2
#define QUEUESIZE         10
#define MAXDATALENGTH     100
#define MAXWAITACKTIME    3000  // Warte bis 3 Sekunden nach ACK/NACK/PR_MISMATCH
#define MAXRETRIESACKWAIT 5     // Nach 5 Mal Timeout wegen ACK/NACK/PR_MISMATCH wird ein Reset ausgeführt
#define MAX_RETRIES       3     // Das Paket wird maximal 3 mal geschickt im Falle vom NACK
#define MAX_WAIT_CYCLES   1000  // Wartet maximal 1000 Zyklen auf Daten vom RS232

// Nutze die UART 1 (Der ATMega128 hat 2 UARTs)
#define USE_UART1

struct dataFromRS232
{
  uint8_t version;                        // 1 Byte. Festkodiert MYPROTOCOLVERSION
  uint8_t deviceFrom;                     // 1 Byte
  uint8_t deviceTo;                       // 1 Byte
  uint8_t packetSize;                     // 1 Byte
  char data[MAXDATALENGTH];               // Befehl + Parameter, bzw. Antwort
  uint8_t parity;                         // 1 Byte
};

#ifdef SOURCE_comm
uint8_t nDataQueue;
struct dataFromRS232 receivedData;
boolean hasDataToProcess;
volatile long timeWaitACK;
uint8_t ackWaitCount;
#endif
#define COMM_LOOPJOB         doLoopJobsComm()

/**
* Funktion um eine komplette Initialisierung des Kommunikationssubsystems
* (über RS232)
*/
void doInitCommunication(void);

/**
* Schließt die Kommunikationen und de-initialisiert das Kommunikationssubsystem
*/
void doCloseCommunication(void);

/**
* Schickt dem Hauptdevice (Laptop) ein "READY" um zu zeigen,
* dass alle Tests erfolgreich durchgelaufen sind.
*/
void sendReadySignal(void);

/**
* Schickt die eingegebenen Daten der Gegenstelle (als erstes schickt ein START)
* @param struct dataFromRS232 toSendData   Die Daten zu schicken
*/
void sendData(struct dataFromRS232 toSendData);

/**
* Fügt einen neuen Datensatz in der Queue
* @param struct dataFromRS232 data   Der Datensatz
* @return                            0, wenn der Datensatz erfolgreich hinzugefüt worden ist, -1 wenn Fehler
*/
uint8_t addToQueue(struct dataFromRS232 data);

/**
* Schickt die Daten in der Queue an den entsprechenden Devices
*/
void sendDataQueue(void);

/**
 * Berechnet die Parität für das empfangenen Packet
 *
 * @param struct dataFromRS232 data  Die empfangenen Daten
 * @return uint8_t                   Die Parität
 */
uint8_t calculateParity(struct dataFromRS232 data);

/**
* Erledigt die Loopjobs der Kommunikationsschnittstelle
* Wenn Daten vom RS232 kommen, bearbeitet sie, verwaltet die ACK
* und, wenn nötig, schickt die Meldungen in der Queue
*/
void doLoopJobsComm(void);

/**
* Wenn Daten vom Hauptgerät (Laptop) empfangen worden sind,
* gibt es einen Zeichen zum verwalten
*
* @return boolean      TRUE wenn Daten zu verwalten gibt, FALSE ansonsten
*/
#define hasToProcessDataFromMainDevice()   (hasDataToProcess == TRUE)

/**
* Meldet, dass alle empfangenen Daten verwaltet worden sind
*/
#define dataIsProcessed()                  (hasDataToProcess = FALSE)

/**
* Gibt die Daten zu verwalten zurück
*
* @return struct dataFromRS232      Die zu bearbeitenen Daten
*/
#define getDataToProcess()                 receivedData
